/*
 *  check.c - checking functions
 *  Copyright (c)2002 - 2004 Andrej  OM1CW
 *  Copyright (c)2005 Mike Karas OM4AA <mike.om4aa@gmail.com>
 *  Copyright (c)2009 Ladislav Vaiz OK1ZIA <ok1zia@nagano.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "osdep.h"
#include "check.h"
#include "main.h"
#include "debug.h"
#include "call.h"
#include "compare.h"
#include "cdwdb.h"


int good_qso(struct QSO_INFO *q)
{
	DEBUG_FUNC("good_qso");

	return (q->pass < 100 && q->ok_call == q->r_call &&
			strcasecmp(q->ok_rst1,q->r_rst1)==0 && 
            q->ok_nr == q->r_nr && q->r_call->loc && q->r_nr && !q->wwl_outside);
}

void make_xchk2(uint8_t band, uint8_t pass, uint8_t mask)
{
	DEBUG_FUNC("make_xchk2");

	struct CALL_INFO *c = first_call[band];
	struct QSO_INFO *q, *r;
	uint32_t p;
//	int entries = 0;
	uint8_t found;

	// printf("+ Xcheck pass #%i: ", pass);

	printf(".");

	while (c) {
		q = c->first_qso;
		if (q) {
			while (q) {
				if (!q->pass) {
					r = q->ok_call->first_qso;
					if (r) {
						found = 0;
						while (r) {
							if (!r->pass) {
//                                    printf("57: ok_call=%s %s\n", q->r_call->call, r->s_call);

								p = 0;
								if (!strcmp(r->ok_call->call, q->s_call->call))
									p += 64;	// he rcvd good CALL
								// if (r->s_call == q->ok_call) p+=64;
								// //ja som prijal dobre CALL
								if (strcasecmp(r->r_rst1,q->s_rst1)==0)
									p += 32;	// he rcvd good RST
								if (strcasecmp(r->s_rst1,q->r_rst1)==0)
									p += 16;	// I rcvd good RST
								if (r->r_nr == q->s_nr)
									p += 8;	// he rcvd good NR
								if (r->s_nr == q->r_nr)
									p += 4;	// I rcvd good NR
								if (r->ok_call->loc == q->s_call->loc)
									p += 2;	// he rcvd good WWL
								if (r->s_call->loc == q->ok_call->loc)
									p += 1;	// I rcvd good WWL

//                                printf("%x %x\n", p, mask);
								if ((p & mask) == mask) {
									found = 1;
//									entries += 2;
									q->ok_qso = r;
									r->ok_qso = q;
/*                                    printf("\n87: q=%s q->ok_qso=%s r=%s r->ok_qso=%s\n", 
                                            q->s_call, q->ok_qso->s_call, r->s_call, r->ok_qso->s_call);
                                    printf("89: %s %s\n", r->ok_call->call, q->s_call->call);*/
//                                  printf("90: ok_call=%s %s\n", q->r_call->call, q->ok_qso->s_call);
//                                    printf("91: ok_call=%s %s\n", r->r_call->call, r->ok_qso->s_call);
									q->pass = pass;
									r->pass = pass;
									break;
								}
							}

							r = r->next;
						}

					}

				}

				q = q->next;
			}
		}
		c = c->next;
	}

	// printf("DONE...%d%% of entries\n", entries*100/all_entries);
}

void make_not_in_log(uint8_t band)
{
	DEBUG_FUNC("make_not_in_log");

	struct CALL_INFO *c = first_call[band];
	struct QSO_INFO *q, *r;
	uint32_t /* entries=0, */ lost = 0;
	uint8_t found;

	// printf("+ Xcheck pass #%i: ", pass);

	printf(".");

	while (c) {
		q = c->first_qso;
		if (q) {
			while (q) {
				if (!q->pass) {
					r = q->ok_call->first_qso;
					if (r) {
						found = 0;
						while (r) {
							if (!strcmp(r->ok_call->call, c->call)) {
								found = 1;
								break;
							}
							r = r->next;
						}

						if (!found) {
							lost++;
							q->pass = 100;
						}
					}

				}

				q = q->next;
			}
		}
		c = c->next;
	}

	// printf("DONE...%d%% of entries\n", entries*100/all_entries);
}

/* check and correct bad calls */
void make_bad_call(uint8_t band)
{
	DEBUG_FUNC("make_bad_call");

	struct CALL_INFO *c = first_call[band];
	struct QSO_INFO *q;
	char text[SIZE_CALL + 1];
	int bad_call = 0;

	while (c) {
		q = c->first_qso;
		while (q) {
			if (q->r_call->refs <= 3) {
//                printf("176: %s q->r_call->refs=%d\n", q->r_call->call, q->r_call->refs);
				if ((bad_call = found_in_bad_calls(q->r_call->call)) != -1) {
					strncpy(text, bad_calls[bad_call].correct, SIZE_CALL);
					if (verbose_mode > 1){
						fprintf(inf, "%s correcting BADCALL: %s to %s!\n", c->call, q->r_call->call, text);
					    //printf("%s correcting BADCALL: %s to %s!\n", c->call, q->r_call->call, text);
                    }
				} else{
					strncpy(text, q->r_call->call, SIZE_CALL);
                }
				if (strcmp(text, q->r_call->call)){
					q->ok_call = add_call(text, q->r_call->loc, band);
//                    printf("180: add %d %d\n", text, q->r_call->loc);
                }
			}
			q = q->next;
		}
		c = c->next;
	}
}

void make_xchk_res(uint8_t band)
{
	DEBUG_FUNC("make_xchk_res");

	struct CALL_INFO *c = first_call[band];
	struct QSO_INFO *q;

	printf(".");

	// printf("+ Xcheck result: ");

	while (c) {
		q = c->first_qso;
		while (q) {
			if (q->ok_qso) {
//                printf("204: ok_call=%s %s\n", q->r_call->call, q->ok_qso->s_call);
				q->ok_call = q->ok_qso->s_call;
				strcpy(q->ok_rst1, q->ok_qso->s_rst1);
				q->ok_nr = q->ok_qso->s_nr;
			}
			q = q->next;
		}
		c = c->next;
	}

	// printf("DONE\n");
}

void make_dupe(uint8_t band)
{
	DEBUG_FUNC("make_dupe");

	struct CALL_INFO *c = first_call[band];
	struct QSO_INFO *q, *q2;

	// printf("+ DUPE check: ");

	printf(".");

	while (c) {
		q = c->first_qso;
		// if (q) printf(".");
		while (q) {
			if (good_qso(q) && !q->dupe) {
				q2 = q->next;
				while (q2) {
					if (compare_calls2(q->ok_call->call, q2->ok_call->call)){
//                        printf("compare_calls2('%s', '%s')\n", q->ok_call->call, q2->ok_call->call);
						q2->pass = 101;
                    }
					q2 = q2->next;
				}
			}
			q = q->next;
		}
		c = c->next;
	}

	// printf("OK\n");
}

void make_outside_wwl(uint8_t band)
{
#ifdef HAVE_GLIB2
	DEBUG_FUNC("make_outside_wwl");
	
	struct CALL_INFO *c = first_call[band];
	struct QSO_INFO *q;
	char loc[20];
	int have_log;
	FILE *fd;
	
	if (!dw)
		return;

	printf(".");
	fd = open_repfile(band, 1);
	fprintf(fd, "\n\nMaking check of outside WWLs:\n");
	while (c) {
		q = c->first_qso;
		have_log = 0;
		while (q) {
            int wwlok=wwl_is_ok_by_call(dw, num2loc(q->ok_call->loc, loc), q->ok_call->call);
            if (!wwlok) {
				if (q->ok_call->first_qso)
					have_log = 1;
				else
					q->wwl_outside = 1;
				if (verbose_mode > 1 && c->categ > 0) {
					fprintf(fd, "%s: %s(%s) is outside of the country", c->call, q->ok_call->call, num2loc(q->ok_call->loc, loc));
					if (have_log)
						fprintf(fd, ", but have log - NOT repaired!");
					fprintf(fd, "\n");
				}
			}
            if (wwlok==2){
                eprintf("%s: can't find dxcc for call %s\n", c->filepath, q->ok_call->call);
            }
			q = q->next;
		}
		c = c->next;
	}
	close_repfile(fd);
#endif
}
